/*=======================================================================
              bpg load plugin for the GIMP
                 Copyright (C) 2014 - Mikro

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
=======================================================================*/

#include <stdint.h>
#include <string.h>

#include <glib.h>
#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include <bpg/libbpg.h>

#include "read-bpg.h"

const char LOAD_PROCEDURE[] = "file-bpg-load";
const char BINARY_NAME[]    = "file-bpg";

void query();
void run(const gchar *, gint, const GimpParam *, gint *, GimpParam **);

// declare the plugin entry points
GimpPlugInInfo PLUG_IN_INFO = {
    NULL,
    NULL,
    query,
    run
};

MAIN()

void query()
{
    // load arguments
    static const GimpParamDef load_arguments[] =
    {
        { GIMP_PDB_INT32,  "run-mode",     "Interactive, non-interactive" },
        { GIMP_PDB_STRING, "filename",     "The name of the file to load" },
        { GIMP_PDB_STRING, "raw-filename", "The name entered" }
    };

    // load return values
    static const GimpParamDef load_return_values[] =
    {
        { GIMP_PDB_IMAGE, "image", "Output image" }
    };

    // install the load procedure
    gimp_install_procedure(LOAD_PROCEDURE,
                           "Import BPG image",
                           "Import BPG image",
                           "Mikro",
                           "Mikro",
                           "2014",
                           "bpg image",
                           NULL,
                           GIMP_PLUGIN,
                           G_N_ELEMENTS(load_arguments),
                           G_N_ELEMENTS(load_return_values),
                           load_arguments,
                           load_return_values);

    // register load handlers
    gimp_register_file_handler_mime(LOAD_PROCEDURE, "image/bpg");
    gimp_register_load_handler(LOAD_PROCEDURE, "bpg", "");
}

// the run function
void run(const gchar * name, gint nparams, const GimpParam * param,
         gint * nreturn_vals, GimpParam ** return_vals) {
  
    // create the return value.
    static GimpParam return_values[2];
    *nreturn_vals = 1;
    *return_vals  = return_values;
    int new_image_id;
    
    // set the return value to success by default
    return_values[0].type          = GIMP_PDB_STATUS;
    return_values[0].data.d_status = GIMP_PDB_SUCCESS;

    // check that this is a load procedure
    if(!strcmp(name, LOAD_PROCEDURE))
    {
      // 3 parameters are needed for the import function
      if(nparams != 3)
      {
	  return_values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
	  return;
      }

      // image loading and processing
      new_image_id = read_bpg(param[1].data.d_string);

      // any error?
      if(new_image_id == -1)
      {
	  return_values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
	  return;
      }

      // fill in the second return value
      *nreturn_vals = 2;

      return_values[1].type         = GIMP_PDB_IMAGE;
      return_values[1].data.d_image = new_image_id;
    }
    else
      return_values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
}
